@@ -6,15 +6,37 @@ from .util_ import procrustes, issparse, sparse_matrix, nearest_neighbors
 
 import scipy
 from scipy.sparse import linalg as slinalg
-from scipy.sparse import csr_matrix, csc_matrix, block_diag, vstack, bmat
+from scipy.sparse import csr_matrix, csc_matrix, lil_matrix, block_diag, vstack, bmat
 from sklearn.neighbors import NearestNeighbors
 from scipy.spatial.distance import pdist, squareform
+from scipy.sparse.csgraph import dijkstra
 
 import multiprocess as mp
 from multiprocess import shared_memory
 
 import scs
 
+def compute_far_off_points(d_e, global_opts, force_compute=False):
+    if 'reuse' in global_opts['far_off_points_type'] and (not force_compute):
+        return global_opts['far_off_points']
+
+    if global_opts['far_off_points_type'] != 'random':
+        np.random.seed(42)
+    far_off_points = []
+    dist_from_far_off_points = None
+    while len(far_off_points) < global_opts['n_repel']:
+        if len(far_off_points) == 0:
+            far_off_points = [np.random.randint(0,d_e.shape[0])]
+            dist_from_far_off_points = dijkstra(d_e, directed=False,
+                                                indices=far_off_points[-1])
+        else:
+            far_off_points.append(np.argmax(dist_from_far_off_points))
+            dist_from_far_off_points = np.minimum(dist_from_far_off_points,
+                                                  dijkstra(d_e, directed=False,
+                                                           indices=far_off_points[-1]))
+    return far_off_points
+    
+
 # Computes Z_s for the case when to_tear is True.
 # Input Z_s is the Z_s for the case when to_tear is False.
 # Output Z_s is a subset of input Z_s.
@@ -109,9 +131,8 @@ def procrustes_init(seq, rho, y, is_visited_view, d, Utilde, n_Utilde_Utilde,
         C_s = C[s,:].indices
         y[C_s,:] = intermed_param.eval_({'view_index': s, 'data_mask': C_s})
     return y, is_visited_view
-    
-# Ngoc-Diep Ho, Paul Van Dooren, On the pseudo-inverse of the Laplacian of a bipartite graph
-def compute_Lpinv_BT(W, B):
+
+def compute_Lpinv_helpers(W):
     M, n = W.shape
     # B_ = W.copy().transpose().astype('int')
     B_ = W.copy().transpose().astype('float')
@@ -135,8 +156,15 @@ def compute_Lpinv_BT(W, B):
     U2 = U12[:,m_1:]
     V1 = V[:,:m_1]
     V2 = V[:,m_1:]
-    
-    B_n = B - B.mean(axis=1)
+    return [D_1_inv_sqrt, D_2_inv_sqrt, U1, U2, V1, V2, Sigma_1, Sigma_2]
+
+def compute_Lpinv_MT(Lpinv_helpers, B):
+    D_1_inv_sqrt, D_2_inv_sqrt, U1, U2, V1, V2, Sigma_1, Sigma_2 = Lpinv_helpers
+    n = D_1_inv_sqrt.shape[0]
+    B_mean = B.mean(axis=1)
+    if len(B_mean.shape) == 1:
+        B_mean = B_mean[:,None]
+    B_n = B - B_mean
     B_n = np.asarray(B_n)
     B1T = D_1_inv_sqrt * (B_n[:, :n].T)
     B2T = D_2_inv_sqrt.T * (B_n[:, n:].T)
@@ -158,6 +186,11 @@ def compute_Lpinv_BT(W, B):
     temp = temp - np.mean(temp, axis=0, keepdims=True)
     return temp
 
+# Ngoc-Diep Ho, Paul Van Dooren, On the pseudo-inverse of the Laplacian of a bipartite graph
+def compute_Lpinv_BT(W, B):
+    D_1_inv_sqrt, D_2_inv_sqrt, U1, U2, V1, V2, Sigma_1, Sigma_2 = compute_Lpinv_helpers(W)
+    return compute_Lpinv_MT(Lpinv_helpers, B)
+
 # conjugate gradient based approach
 def compute_Lpinv_BT_cg(Utilde, B):
     pdb.set_trace()
@@ -176,7 +209,7 @@ def compute_CC(D, B, Lpinv_BT):
     CC = D - B.dot(Lpinv_BT)
     return 0.5*(CC + CC.T)
 
-def build_ortho_optim(d, Utilde, intermed_param, ret_D=False,
+def build_ortho_optim(d, Utilde, intermed_param,
                       far_off_points=[], repel_by=0.,
                       beta=None):
     M,n = Utilde.shape
@@ -185,54 +218,62 @@ def build_ortho_optim(d, Utilde, intermed_param, ret_D=False,
     B_vals = []
     D = []
     
-    W_row_inds = []
-    W_col_inds = []
-    W_vals = []
+    if (beta is not None) and (beta['align'] is not None):
+        W_row_inds = []
+        W_col_inds = []
+        W_vals = []
+        for i in range(M):
+            Utilde_i = Utilde[i,:].indices
+            w_ki = intermed_param.alignment_wts({'view_index': i,
+                                                  'data_mask': Utilde_i,
+                                                  'beta': beta['align']})
+            col_inds = Utilde_i.tolist()
+            W_row_inds += [i]*len(col_inds)
+            W_col_inds += col_inds
+            W_vals += w_ki.tolist()
+        W_row_inds = np.array(W_row_inds)
+        W_col_inds = np.array(W_col_inds)
+        W_vals = np.array(W_vals)
+        for k in range(n):
+            mask = W_col_inds == k
+            temp = W_vals[mask]
+            temp = np.exp(temp - temp.max())
+            temp *= temp.shape[0]/np.sum(temp)
+            W_vals[mask] = temp
+        W = csr_matrix((W_vals, (W_row_inds, W_col_inds)), shape=(M,n), dtype=float)
+    else:
+        W = Utilde.astype(float)
+        W_vals = W.data
+        
     
     for i in range(M):
         Utilde_i = Utilde[i,:].indices
         X_ = intermed_param.eval_({'view_index': i,
                                    'data_mask': Utilde_i})
-        
-        w = None
-        if beta:
-            anom_scores = intermed_param.anom_score_({'view_index': i, 'data_mask': Utilde_i})
-            if anom_scores is not None:
-                w = np.exp(-anom_scores/beta)
-        
-        if w is None:
-            w = np.ones((Utilde_i.shape[0]))
-        
-        X_ = w[:,None] * X_
-        
+        sqrt_p_ki = np.sqrt(np.array(W[i,:].data).flatten()[:,None])
+        X_ = sqrt_p_ki * X_
         D.append(np.matmul(X_.T,X_))
         
         row_inds = list(range(d*i,d*(i+1)))
         col_inds = Utilde_i.tolist()
         
-        W_row_inds += [i]*len(col_inds)
-        W_col_inds += col_inds
-        W_vals += (w**2).tolist()
-        
         B_row_inds += (row_inds + np.repeat(row_inds, len(col_inds)).tolist())
         B_col_inds += (np.repeat([n+i], d).tolist() + np.tile(col_inds, d).tolist())
+        
+        X_ = sqrt_p_ki * X_
         B_vals += (np.sum(-X_.T, axis=1).tolist() + X_.T.flatten().tolist())
     
     D = block_diag(D, format='csr')
     B = csr_matrix((B_vals, (B_row_inds, B_col_inds)), shape=(M*d,n+M))
-    W = csr_matrix((W_vals, (W_row_inds, W_col_inds)), shape=(M,n), dtype=float)
     
-    if beta:
-        print('min and max anom scores:', (-beta*np.log(np.array(W_vals))).min(),
-                                          (-beta*np.log(np.array(W_vals))).max())
     print('min and max weights:', np.array(W_vals).min(), np.array(W_vals).max())
 
+    n_repel = len(far_off_points)
     print('Computing Pseudoinverse of a matrix of L of size', n, '+', M, 'multiplied with B', flush=True)
-    Lpinv_BT = compute_Lpinv_BT(W, B)
-
+    Lpinv_helpers = compute_Lpinv_helpers(W)
+    Lpinv_BT = compute_Lpinv_MT(Lpinv_helpers, B)
     CC = compute_CC(D, B, Lpinv_BT)
     
-    n_repel = len(far_off_points)
     if n_repel > 0:
         L__row_inds = []
         L__col_inds = []
@@ -241,21 +282,29 @@ def build_ortho_optim(d, Utilde, intermed_param, ret_D=False,
         for i in range(n_repel):
             L__row_inds += [far_off_points[i]]*n_repel
             L__col_inds += far_off_points
-            temp_arr[i] = n_repel
-            L__vals += temp_arr
-            temp_arr[i] = -1
+            if (beta is not None) and (beta['repel'] is not None):
+                p_llp = intermed_param.repulsion_wts({'pt_index': far_off_points[i],
+                                                      'repelling_pts_indices': far_off_points,
+                                                      'beta': beta['repel']})
+                p_llp_sum = np.sum(p_llp)
+                p_llp *= -1
+                p_llp[i] += p_llp_sum
+                L__vals += p_llp.tolist()
+            else:
+                temp_arr[i] = n_repel-1
+                L__vals += temp_arr
+                temp_arr[i] = -1
+                
         L_ = csr_matrix((L__vals, (L__row_inds, L__col_inds)), shape=(n+M,n+M))
         L_ = -repel_by*L_
-        CC = CC + (Lpinv_BT.T).dot(L_.dot(Lpinv_BT))
+        L__Lpinv_BT = L_.dot(Lpinv_BT)
+        CC = CC + (Lpinv_BT.T).dot(L__Lpinv_BT)
+        Lpinv_BT -= compute_Lpinv_MT(Lpinv_helpers, L__Lpinv_BT.T)
+    return CC, Lpinv_BT, D, B
     
-    if ret_D:
-        return CC, Lpinv_BT, D
-    else:
-        return CC, Lpinv_BT
-    
-
+# unscaled alignment error
 def compute_alignment_err(d, Utilde, intermed_param, scale_num, far_off_points=[], repel_by=0., beta=None):
-    CC, Lpinv_BT = build_ortho_optim(d, Utilde, intermed_param,
+    CC, Lpinv_BT, _, _ = build_ortho_optim(d, Utilde, intermed_param,
                                      far_off_points=far_off_points,
                                      repel_by=repel_by, beta=beta)
     M,n = Utilde.shape
@@ -267,8 +316,9 @@ def compute_alignment_err(d, Utilde, intermed_param, scale_num, far_off_points=[
     #print('Smallest singular value of C', sigma_min_C, flush=True)
     
     CC_mask = np.tile(np.eye(d, dtype=bool), (M,M))
-    scale_denom = Utilde.sum()
-    scale = (scale_num/scale_denom)**2
+    #scale_denom = Utilde.sum()
+    #scale = (scale_num/scale_denom)
+    scale = 1
     err = np.sum(CC[CC_mask]) * scale
     return err
 
@@ -277,7 +327,7 @@ def compute_alignment_err(d, Utilde, intermed_param, scale_num, far_off_points=[
 def spectral_alignment(y, is_visited_view, d, Utilde,
                       C, intermed_param, global_opts, 
                       seq_of_intermed_views_in_cluster):
-    CC, Lpinv_BT = build_ortho_optim(d, Utilde, intermed_param,
+    CC, Lpinv_BT, _, _ = build_ortho_optim(d, Utilde, intermed_param,
                                      far_off_points=global_opts['far_off_points'],
                                      repel_by=global_opts['repel_by'],
                                      beta=global_opts['beta'])
@@ -288,10 +338,17 @@ def spectral_alignment(y, is_visited_view, d, Utilde,
     print('Computing eigh(C,k=d)', flush=True)
     np.random.seed(42)
     v0 = np.random.uniform(0,1,CC.shape[0])
-    # To find smallest eigenvalues, using shift-inverted algo with mode=normal and which='LM'
-    W_,V_ = scipy.sparse.linalg.eigsh(CC, k=d, v0=v0, sigma=0.0)
-    # or just pass which='SM' without using sigma
-    # W_,V_ = scipy.sparse.linalg.eigsh(CC, k=d, v0=v0, which='SM')
+    if (global_opts['n_repel'] == 0) or (global_opts['repel_by'] == 0):
+        # To find smallest eigenvalues, using shift-inverted algo with mode=normal and which='LM'
+        W_,V_ = scipy.sparse.linalg.eigsh(CC, k=d, v0=v0, sigma=0.0)
+        # or just pass which='SM' without using sigma
+        # W_,V_ = scipy.sparse.linalg.eigsh(CC, k=d, v0=v0, which='SM')
+    else:
+        # To find smallest eigenvalues, using shift-inverted algo with mode=normal and which='LM'
+        CC_frob = np.linalg.norm(CC)
+        W_,V_ = scipy.sparse.linalg.eigsh(CC, k=d, v0=v0, sigma=-2*CC_frob)
+        # or just pass which='SM' without using sigma
+        # W_,V_ = scipy.sparse.linalg.eigsh(CC, k=d, v0=v0, which='SA')
     print('Done.', flush=True)
     Wstar = np.sqrt(M)*V_.T
     Tstar = np.zeros((d, M*d))
@@ -318,7 +375,6 @@ def spectral_alignment(y, is_visited_view, d, Utilde,
     for i in range(n_clusters):
         seq = seq_of_intermed_views_in_cluster[i]
         s0 = seq[0]
-        print(Zstar[:,n+s0])
         Zstar[:,n+seq] -= Zstar[:,n+s0][:,None]
     
     for i in range(n_clusters):
@@ -407,7 +463,7 @@ def rgd_final(y, d, Utilde, C, intermed_param,
              parents_of_intermed_views_in_cluster,
              cluster_of_intermed_view,
              global_opts):
-    CC, Lpinv_BT = build_ortho_optim(d, Utilde, intermed_param,
+    CC, Lpinv_BT, _, _ = build_ortho_optim(d, Utilde, intermed_param,
                                      far_off_points=global_opts['far_off_points'],
                                      repel_by=global_opts['repel_by'],
                                      beta=global_opts['beta'])
@@ -508,18 +564,18 @@ def rgd_final(y, d, Utilde, C, intermed_param,
         intermed_param.v[s,:] = np.matmul(intermed_param.v[s,:][np.newaxis,:], T_s) + v_s
         C_s = C[s,:].indices
         y[C_s,:] = intermed_param.eval_({'view_index': s, 'data_mask': C_s})
-
     return y
 
+
 def gpm_final(y, d, Utilde, C, intermed_param,
              n_Utilde_Utilde, n_Utildeg_Utildeg,
              parents_of_intermed_views_in_cluster,
              cluster_of_intermed_view,
              global_opts):
-    CC, Lpinv_BT, D = build_ortho_optim(d, Utilde, intermed_param, ret_D=True,
-                                        far_off_points=global_opts['far_off_points'],
-                                        repel_by=global_opts['repel_by'],
-                                        beta=global_opts['beta'])
+    CC, Lpinv_BT, D, _ = build_ortho_optim(d, Utilde, intermed_param,
+                                            far_off_points=global_opts['far_off_points'],
+                                            repel_by=global_opts['repel_by'],
+                                            beta=global_opts['beta'])
     CC = D - CC
     M,n = Utilde.shape
     n_proc = min(M,global_opts['n_proc'])
@@ -621,7 +677,7 @@ def sdp_alignment(y, is_visited_view, d, Utilde,
                   C, intermed_param, global_opts, 
                   seq_of_intermed_views_in_cluster,
                   solver=None):
-    CC, Lpinv_BT = build_ortho_optim(d, Utilde, intermed_param,
+    CC, Lpinv_BT, _, _ = build_ortho_optim(d, Utilde, intermed_param,
                                      far_off_points=global_opts['far_off_points'],
                                      repel_by=global_opts['repel_by'],
                                      beta=global_opts['beta'])
