@@ -4,7 +4,7 @@ import numpy as np
 import copy
 
 from .util_ import procrustes, print_log, nearest_neighbors, sparse_matrix, lexargmax
-from .global_reg_ import procrustes_init, spectral_alignment, sdp_alignment, procrustes_final, rgd_final, gpm_final, compute_alignment_err
+from .global_reg_ import procrustes_init, spectral_alignment, sdp_alignment, procrustes_final, rgd_final, gpm_final, compute_alignment_err, compute_far_off_points
 
 from scipy.linalg import svdvals
 from scipy.sparse.csgraph import minimum_spanning_tree, breadth_first_order
@@ -46,8 +46,9 @@ class GlobalViews:
                                               self.local_start_time, 
                                               self.global_start_time)
             
-    def fit(self, d, Utilde, C, c, n_C, intermed_param, global_opts, vis, vis_opts):
-        if global_opts['main_algo'] == 'LDLE':
+    def fit(self, d, d_e, Utilde, C, c, n_C, intermed_param, global_opts, vis, vis_opts):
+        print('Using', global_opts['align_transform'], 'transforms for alignment.')
+        if global_opts['align_transform'] == 'rigid':
             # Compute |Utilde_{mm'}|
             n_Utilde_Utilde = Utilde.dot(Utilde.transpose())
             n_Utilde_Utilde.setdiag(0)
@@ -69,7 +70,7 @@ class GlobalViews:
                                   vis_opts, title='Before_Init')
             
             # Compute initial embedding
-            y_init, color_of_pts_on_tear_init = self.compute_init_embedding(d, Utilde, n_Utilde_Utilde, intermed_param,
+            y_init, color_of_pts_on_tear_init = self.compute_init_embedding(d, d_e, Utilde, n_Utilde_Utilde, intermed_param,
                                                                             seq_of_intermed_views_in_cluster,
                                                                             parents_of_intermed_views_in_cluster,
                                                                             C, c, vis, vis_opts, global_opts)
@@ -79,7 +80,7 @@ class GlobalViews:
             
             if global_opts['refine_algo_name']:
                 y_final,\
-                color_of_pts_on_tear_final = self.compute_final_embedding(y_init, d, Utilde, C, intermed_param,
+                color_of_pts_on_tear_final = self.compute_final_embedding(y_init, d, d_e, Utilde, C, c, intermed_param,
                                                                           n_Utilde_Utilde,
                                                                           seq_of_intermed_views_in_cluster,
                                                                           parents_of_intermed_views_in_cluster, 
@@ -88,8 +89,9 @@ class GlobalViews:
                 self.y_final = y_final
                 self.color_of_pts_on_tear_final = color_of_pts_on_tear_final
             
-        elif global_opts['main_algo'] == 'LTSA':
-            print('Using LTSA Global Alignment...')
+        elif global_opts['align_transform'] == 'affine':
+            # TODO
+            print('TODO: Need to implement', flush = True)
 #             self.y_final = self.compute_final_global_embedding_ltsap_based()
 #             self.color_of_pts_on_tear_final = None
         
@@ -99,6 +101,7 @@ class GlobalViews:
             self.n_Utilde_Utilde = n_Utilde_Utilde
             self.seq_of_intermed_views_in_cluster = seq_of_intermed_views_in_cluster
             self.parents_of_intermed_views_in_cluster = parents_of_intermed_views_in_cluster
+            self.cluster_of_intermed_view = cluster_of_intermed_view
     
     # Motivated from graph lateration
     def compute_seq_of_intermediate_views(self, Utilde, n_C, n_Utilde_Utilde,
@@ -123,11 +126,11 @@ class GlobalViews:
             W_data_ = np.zeros(end_ind-start_ind)
             for i in range(start_ind, end_ind):
                 m = W_rows[i]
-                mp = W_cols[i]
-                Utilde_mmp = Utilde[m,:].multiply(Utilde[mp,:]).nonzero()[1]
+                mpp = W_cols[i]
+                Utilde_mmp = Utilde[m,:].multiply(Utilde[mpp,:]).nonzero()[1]
                 # Compute V_{mm'}, V_{m'm}, Vbar_{mm'}, Vbar_{m'm}
                 V_mmp = intermed_param.eval_({'view_index': m, 'data_mask': Utilde_mmp})
-                V_mpm = intermed_param.eval_({'view_index': mp, 'data_mask': Utilde_mmp})
+                V_mpm = intermed_param.eval_({'view_index': mpp, 'data_mask': Utilde_mmp})
                 Vbar_mmp = V_mmp - np.mean(V_mmp,0)[np.newaxis,:]
                 Vbar_mpm = V_mpm - np.mean(V_mpm,0)[np.newaxis,:]
                 # Compute ambiguity as the minimum singular value of
@@ -192,11 +195,17 @@ class GlobalViews:
                parents_of_intermed_views_in_cluster,\
                cluster_of_intermed_view
     
-    def compute_pwise_dist_in_embedding(self, s_d_e, y, Utilde, C, global_opts,
-                                     n_Utilde_Utilde, n_Utildeg_Utildeg=None):
+    # dist = y_d_e
+    def compute_pwise_dist_in_embedding(self, s_d_e, Utilde, C, global_opts,
+                                     n_Utilde_Utilde=None, n_Utildeg_Utildeg=None, y=None, dist=None,):
         M,n = Utilde.shape
-        dist = squareform(pdist(y))
+        
+        if dist is None:
+            dist = squareform(pdist(y))
 
+        if n_Utilde_Utilde is None:
+            n_Utilde_Utilde = Utilde.dot(Utilde.transpose())
+            n_Utilde_Utilde.setdiag(0)
         # Compute |Utildeg_{mm'}| if not provided
         if n_Utildeg_Utildeg is None:
             _, n_Utildeg_Utildeg = self.compute_Utildeg(y, Utilde, C, global_opts)
@@ -205,6 +214,10 @@ class GlobalViews:
         # is connected to jth view if they are neighbors in the
         # ambient space but not in the embedding space
         tear = n_Utilde_Utilde-n_Utilde_Utilde.multiply(n_Utildeg_Utildeg)
+        # no tear then resturn dist
+        if np.sum(tear)==0:
+            return dist
+        
         # Keep track of visited views across clusters of manifolds
         is_visited = np.zeros(M, dtype=bool)
         n_visited = 0
@@ -223,13 +236,13 @@ class GlobalViews:
                     # on the opposite side of the tear
                     Utilde_m = Utilde[m,:]
                     for i in range(len(to_tear_mth_view_with)):
-                        mp = to_tear_mth_view_with[i]
-                        temp_i = Utilde_m.multiply(Utilde[mp,:])
+                        mpp = to_tear_mth_view_with[i]
+                        temp_i = Utilde_m.multiply(Utilde[mpp,:])
                         # Compute points on the overlap of m and m'th view
                         # which are in mth cluster and in m'th cluster. If
                         # both sets are non-empty then assign them same color.
                         temp_m = C[m,:].multiply(temp_i).nonzero()[1]
-                        temp_mp = C[mp,:].multiply(temp_i).nonzero()[1]
+                        temp_mp = C[mpp,:].multiply(temp_i).nonzero()[1]
                         dist[np.ix_(temp_m,temp_mp)] = s_d_e[np.ix_(temp_m,temp_mp)]
                         dist[np.ix_(temp_mp,temp_m)] = s_d_e[np.ix_(temp_mp,temp_m)]
                         pts_on_tear[temp_m] = True
@@ -239,16 +252,43 @@ class GlobalViews:
         # contracting dist.T, dist with min as the binary operation
         print('Computing min(original dist, min(one-hop distances))', flush=True)
         print('#pts on tear', np.sum(pts_on_tear))
-        print_freq = int(n/10)
-        for i in range(n):
-            if np.mod(i, print_freq) == 0:
-                print('Processed', i, 'points.', flush=True)
-            dist[i,:] = np.minimum(dist[i,:],np.min(dist[pts_on_tear,:] + dist[i,pts_on_tear][:,None], axis=0))
+#         print_freq = int(n/10)
+#         for i in range(n):
+#             if np.mod(i, print_freq) == 0:
+#                 print('Processed', i, 'points.', flush=True)
+#             dist[i,:] = np.minimum(dist[i,:],np.min(dist[pts_on_tear,:] + dist[i,pts_on_tear][:,None], axis=0))
+        
+        n_proc = global_opts['n_proc']
+        chunk_sz = int(n/n_proc)
+        def target_proc(p_num, q_, dist, pts_on_tear):
+            start_ind = p_num*chunk_sz
+            if p_num == (n_proc-1):
+                end_ind = n
+            else:
+                end_ind = (p_num+1)*chunk_sz
+            dist_ = np.zeros((end_ind-start_ind,n))
+            for i in range(start_ind, end_ind):
+                dist_[i-start_ind,:] = np.minimum(dist[i,:],np.min(dist[pts_on_tear,:] + dist[i,pts_on_tear][:,None], axis=0))
+            q_.put((start_ind, end_ind, dist_))
+        
+        q_ = mp.Queue()
+        proc = []
+        for p_num in range(n_proc):
+            proc.append(mp.Process(target=target_proc, args=(p_num,q_, dist, pts_on_tear)))
+            proc[-1].start()
+        
+        for p_num in range(n_proc):
+            start_ind, end_ind, dist_ = q_.get()
+            dist[start_ind:end_ind,:] = dist_
+        q_.close()
+        
+        for p_num in range(n_proc):
+            proc[p_num].join()
 
         return dist
     
     def compute_color_of_pts_on_tear(self, y, Utilde, C, global_opts,
-                                     n_Utilde_Utilde, n_Utildeg_Utildeg=None):
+                                        n_Utilde_Utilde, n_Utildeg_Utildeg=None):
         M,n = Utilde.shape
 
         # Compute |Utildeg_{mm'}| if not provided
@@ -272,7 +312,7 @@ class GlobalViews:
             seq, rho = breadth_first_order(n_Utilde_Utilde, s0, directed=False) #(ignores edge weights)
             is_visited[seq] = True
             n_visited = np.sum(is_visited)
-
+            
             # Iterate over views
             for m in seq:
                 to_tear_mth_view_with = tear[m,:].nonzero()[1].tolist()
@@ -281,21 +321,20 @@ class GlobalViews:
                     # on the opposite side of the tear
                     Utilde_m = Utilde[m,:]
                     for i in range(len(to_tear_mth_view_with)):
-                        mp = to_tear_mth_view_with[i]
-                        temp_i = Utilde_m.multiply(Utilde[mp,:])
-                        # Compute points on the overlap of m and m'th view
-                        # which are in mth cluster and in m'th cluster. If
-                        # both sets are non-empty then assign them same color.
+                        mpp = to_tear_mth_view_with[i]
                         temp0 = np.isnan(color_of_pts_on_tear)
+                        temp_i = Utilde_m.multiply(Utilde[mpp,:])
                         temp_m = C[m,:].multiply(temp_i).multiply(temp0)
-                        temp_mp = C[mp,:].multiply(temp_i).multiply(temp0)
-                        if temp_m.sum() and temp_mp.sum():
-                            color_of_pts_on_tear[(temp_m+temp_mp).nonzero()[1]] = cur_color
+                        temp_mp = C[mpp,:].multiply(temp_i).multiply(temp0)
+                        if temp_m.sum():
+                            color_of_pts_on_tear[(temp_m).nonzero()[1]] = cur_color
+                            cur_color += 1
+                        if temp_mp.sum():
+                            color_of_pts_on_tear[(temp_mp).nonzero()[1]] = cur_color
                             cur_color += 1
-                        
         return color_of_pts_on_tear
     
-    def vis_embedding_(self, d, intermed_param, C, Utilde,
+    def vis_embedding_(self, d, intermed_param, c, C, Utilde,
                       n_Utilde_Utilde, global_opts, vis,
                       vis_opts, title='', color_of_pts_on_tear=None,
                       contrib_of_view=None):
@@ -324,9 +363,42 @@ class GlobalViews:
         else:
             color_of_pts_on_tear = None
             
-        vis.global_embedding(y, vis_opts['c'], vis_opts['cmap_interior'],
-                              color_of_pts_on_tear, vis_opts['cmap_boundary'],
-                              title)
+        
+        if color_of_pts_on_tear is not None:
+            pts_on_tear = np.nonzero(~np.isnan(color_of_pts_on_tear))[0]
+            y_ = []
+            ind_ = []
+            #color_of_pts_on_tear = np.zeros(n)+np.nan
+            color_of_pts_on_tear_ = []
+            for i in range(pts_on_tear.shape[0]):
+                k = pts_on_tear[i]
+                for m in Utilde[:,k].nonzero()[0].tolist():
+                    if m == c[k]:
+                        continue
+                    y_.append(intermed_param.eval_({'view_index': m, 'data_mask': np.array([k])}))
+                    ind_.append(k)
+                    color_of_pts_on_tear_.append(color_of_pts_on_tear[k])
+            ind_ = np.array(ind_)
+            color_of_pts_on_tear_ = np.array(color_of_pts_on_tear_)
+            if len(y_):
+                y_ = np.concatenate(y_, axis=0)
+                c_ = vis_opts['c'][ind_]
+                color_of_pts_on_tear_ = np.concatenate([color_of_pts_on_tear, color_of_pts_on_tear_], axis=0)
+            
+                # comment the following three lines if only the boundary is needed
+                y_ = np.concatenate([y,y_], axis=0)
+                c_ = np.concatenate([vis_opts['c'],c_], axis=0)
+                vis.global_embedding(y_,c_, vis_opts['cmap_interior'],
+                                      color_of_pts_on_tear_, vis_opts['cmap_boundary'],
+                                      title)
+            else:
+                vis.global_embedding(y, vis_opts['c'], vis_opts['cmap_interior'],
+                                  color_of_pts_on_tear, vis_opts['cmap_boundary'],
+                                  title)
+        else:
+            vis.global_embedding(y, vis_opts['c'], vis_opts['cmap_interior'],
+                                  color_of_pts_on_tear, vis_opts['cmap_boundary'],
+                                  title)
         plt.show()
         return color_of_pts_on_tear, y
     
@@ -371,7 +443,7 @@ class GlobalViews:
             # rightmost point of the current cluster
             offset = np.max(y[pts_in_cluster_i,0])
     
-    def compute_init_embedding(self, d, Utilde, n_Utilde_Utilde, intermed_param,
+    def compute_init_embedding(self, d, d_e, Utilde, n_Utilde_Utilde, intermed_param,
                                seq_of_intermed_views_in_cluster,
                                parents_of_intermed_views_in_cluster,
                                C, c, vis, vis_opts, global_opts,
@@ -384,6 +456,7 @@ class GlobalViews:
         y = np.zeros((n,d))
 
         n_clusters = len(seq_of_intermed_views_in_cluster)
+        global_opts['far_off_points'] = compute_far_off_points(d_e, global_opts, force_compute=True)
 
         # Boolean array to keep track of already visited views
         is_visited_view = np.zeros(M, dtype=bool)
@@ -434,10 +507,10 @@ class GlobalViews:
                                 intermed_param, C)
         
         # Visualize the initial embedding
-        color_of_pts_on_tear, y = self.vis_embedding_(d, intermed_param, C, Utilde,
+        color_of_pts_on_tear, y = self.vis_embedding_(d, intermed_param, c, C, Utilde,
                                                   n_Utilde_Utilde, global_opts, vis,
                                                   vis_opts, title='Init')
-
+        intermed_param.y = y
         return y, color_of_pts_on_tear
 
     def compute_Utildeg(self, y, Utilde, C, global_opts):
@@ -454,11 +527,11 @@ class GlobalViews:
         n_Utildeg_Utildeg.setdiag(False)
         return Utildeg, n_Utildeg_Utildeg
 
-    def compute_final_embedding(self, y, d, Utilde, C, intermed_param, n_Utilde_Utilde,
+    def compute_final_embedding(self, y, d, d_e, Utilde, C, c, intermed_param, n_Utilde_Utilde,
                                 seq_of_intermed_views_in_cluster,
                                 parents_of_intermed_views_in_cluster, 
                                 cluster_of_intermed_view, global_opts,
-                                vis, vis_opts):
+                                vis, vis_opts, reset=True):
         M,n = Utilde.shape
         y = y.copy()
         n_clusters = len(seq_of_intermed_views_in_cluster)
@@ -469,13 +542,6 @@ class GlobalViews:
         np.random.seed(42) # for reproducbility
 
         old_time = time.time()
-        # If to tear the closed manifolds
-        if global_opts['to_tear']:
-            # Compute |Utildeg_{mm'}|
-            _, n_Utildeg_Utildeg = self.compute_Utildeg(y, Utilde, C, global_opts)
-        else:
-            n_Utildeg_Utildeg = None
-        
         
         CC = None
         Lpinv_BT = None
@@ -483,18 +549,40 @@ class GlobalViews:
         max_iter0 = global_opts['max_iter']
         max_iter1 = global_opts['max_internal_iter']
         refine_algo = global_opts['refine_algo_name']
-        
-        self.tracker['refine_iter_start_at'] = np.zeros(max_iter0)
-        self.tracker['refine_iter_done_at'] = np.zeros(max_iter0)
-        self.tracker['refine_err_at_iter'] = np.zeros(max_iter0)
-        
+        patience_ctr = global_opts['patience']
+        err_tol = global_opts['err_tol']
+        prev_err = None
         contrib_of_view = Utilde.copy()
         solver = None
+        
+        if reset:
+            self.y_spec_refined_y_2 = []
+            self.tracker['refine_iter_start_at'] = []
+            self.tracker['refine_iter_done_at'] = []
+            self.tracker['refine_err_at_iter'] = []
+            self.tracker['|E(Gamma_t)|'] = []
+            self.it0 = 0
+            self.refinement_converged = False
+        else:
+            self.log('Reset is False. Starting from where left off', log_time=True)
+            if self.refinement_converged:
+                self.log('Refinement had already converged.', log_time=True)
+                return self.y_final, self.color_of_pts_on_tear_final
+        
+        # If to tear the closed manifolds
+        if global_opts['to_tear']:
+            # Compute |Utildeg_{mm'}|
+            _, n_Utildeg_Utildeg = self.compute_Utildeg(y, Utilde, C, global_opts)
+        else:
+            n_Utildeg_Utildeg = None
+        
         # Refine global embedding y
         for it0 in range(max_iter0):
-            self.tracker['refine_iter_start_at'][it0] = time.time()
+            self.tracker['refine_iter_start_at'].append(time.time())
             self.log('Refining with ' + refine_algo + ' algorithm for ' + str(max_iter1) + ' iterations.')
-            self.log('Refinement iteration: %d' % it0, log_time=True)
+            self.log('Refinement iteration: %d' % self.it0, log_time=True)
+            
+            global_opts['far_off_points'] = compute_far_off_points(d_e, global_opts)
             
             if global_opts['to_tear']:
                 # Compute which points contribute to which views
@@ -563,7 +651,7 @@ class GlobalViews:
                                        solver=solver)
                 
             self.log('Done.', log_time=True)
-            self.tracker['refine_iter_done_at'][it0] = time.time()
+            self.tracker['refine_iter_done_at'].append(time.time())
 
             if global_opts['compute_error'] or (it0 == max_iter0-1):
                 self.log('Computing error.')
@@ -572,7 +660,14 @@ class GlobalViews:
                                             repel_by=global_opts['repel_by'],
                                             beta=global_opts['beta'])
                 self.log('Alignment error: %0.6f' % err, log_time=True)
-                self.tracker['refine_err_at_iter'][it0] = err
+                self.tracker['refine_err_at_iter'].append(err)
+                self.tracker['|E(Gamma_t)|'].append(contrib_of_view.sum())
+                if prev_err is not None:
+                    if np.abs(err-prev_err) < err_tol:
+                        patience_ctr -= 1
+                    else:
+                        patience_ctr = global_opts['patience']
+                prev_err = err
                 
             self.add_spacing_bw_clusters(d, seq_of_intermed_views_in_cluster,
                                          intermed_param, C)
@@ -590,13 +685,24 @@ class GlobalViews:
             if self.debug:
                 self.y_refined_at.append(y)
                 self.color_of_pts_on_tear_at.append(color_of_pts_on_tear)
+                if refine_algo == 'spectral':
+                    self.y_spec_refined_y_2.append(y_2)
                 
              
             # Visualize the current embedding
-            _, y = self.vis_embedding_(d, intermed_param, C, Utilde,
+            _, y = self.vis_embedding_(d, intermed_param, c, C, Utilde,
                               n_Utilde_Utilde, global_opts, vis,
                               vis_opts, title='Iter_%d' % it0,
                               color_of_pts_on_tear=color_of_pts_on_tear,
                               contrib_of_view=contrib_of_view)
-
+            intermed_param.y = y
+            # visualize the alternate embedding by spectral method
+            if refine_algo == 'spectral':
+                self.vis_embedding(y_2, vis, vis_opts,
+                                   color_of_pts_on_tear=color_of_pts_on_tear,
+                                   title='Alt. embed. Iter_%d' % it0)
+            self.it0 += 1
+            if global_opts['compute_error'] and (patience_ctr <= 0):
+                self.refinement_converged = True
+                break
         return y , color_of_pts_on_tear
\ No newline at end of file
